unit FM_INSTALL;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, checklst, Gauges, IBInstall, ShlObj, ActiveX, ComObj,
  Registry, Winsvc, iblicense, ExtCtrls, ShellAPI ;

type
  TFRM_INSTALL = class(TForm)
    InstallProgressGauge: TGauge;
    TMB_BASLA: TTimer;
    procedure TMB_BASLATimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    OptionsHandle: OPTIONS_HANDLE;
    SourceDir: String;
    RebootMessage: Boolean;
    Cancelling: Boolean;

    function DisplayErrorMessage(handle: OPTIONS_HANDLE; error: MSG_NO): word;
    function ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd: Integer): THandle;

  public
    { Public declarations }
  end;
  function DisplayError( msg: MSG_NO; data: pointer; error_msg: TEXT): integer; export; stdcall;
  function DisplayStatus(status: integer; data: pointer; const status_msg: TEXT): integer; export; stdcall;

var
  FRM_INSTALL: TFRM_INSTALL;
  VAR_CALISTIRILACAK_DOSYA_ADI:string;

const
  MaxComp = 8;
  ComponentOptionNumber: array [0..MaxComp-1] of OPT =(IB_SERVER, IB_CLIENT, IB_DOC, IB_CMD_TOOLS, IB_GUI_TOOLS,
                                                        IB_DEV, IB_EXAMPLE_API, IB_EXAMPLE_DB);


implementation

{$R *.DFM}

function TFRM_INSTALL.ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil, StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;

function TFRM_INSTALL.DisplayErrorMessage(handle: OPTIONS_HANDLE; error: MSG_NO): word;
var
  msg: PChar;
begin
  msg := StrAlloc(ISC_INSTALL_MAX_MESSAGE_LEN);
  isc_install_get_message(handle, error, msg, ISC_INSTALL_MAX_MESSAGE_LEN);
  if (error < isc_install_success) then
    result := MessageDlg(Strpas(msg),mtWarning,[mbOK, mbIgnore],0)
  else
    result := MessageDlg(Strpas(msg),mtWarning,[mbOK],0);
  StrDispose(msg);
end;


function DisplayError(msg: MSG_NO; data: pointer; error_msg: TEXT): integer;
var
  retval: word;
begin
  if (not FRM_INSTALL.Cancelling) then
  begin
    retval := mrIgnore;
    if (msg = isc_install_fp_copy_delayed) or (msg = isc_install_fp_delete_delayed) then
      FRM_INSTALL.RebootMessage := true
    else
      retval := FRM_INSTALL.DisplayErrorMessage(FRM_INSTALL.OptionsHandle, msg);
    if retval = mrIgnore then
      result := isc_install_fp_continue
    else
    begin
      FRM_INSTALL.Cancelling := true;
      result := isc_install_fp_abort;
    end;
  end
  else
    result := isc_install_fp_abort;
end;

function DisplayStatus(status: integer; data: pointer; const status_msg: TEXT): integer;
begin
  FRM_INSTALL.InstallProgressGauge.Progress := status;
  Application.ProcessMessages;
  result := isc_install_fp_continue;
end;

procedure TFRM_INSTALL.TMB_BASLATimer(Sender: TObject);
var
  err_no: MSG_NO;
  TmpDestDir: PChar;
  i: Integer;
  VAR_DESTINATION_DIR, VAR_UNINSTALL_FILE_NAME:string;
  ret_val: word;
  UninstallFilename: PChar;
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  verInfo: TOSVersionInfo;
  ServiceArgs: PChar;
  StartupInfo: TStartupInfo;
  ProcessInformation: TProcessInformation;
  retval   : longint;
  regHndl  : HKey;
  security : REGSAM;
  KeyMade  : Dword;
  UninstallString: String;
const
  GUARDIAN_SERVICE_NAME = 'InterBaseGuardian';
  GUARDIAN_APPLICATION_NAME = '%s\bin\ibguard.exe';
  RegUninstallKey = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\InterBase';
  RegUninstallName = 'InterBase 6.01';
  RegUninstallString = '"%s\IB_Uninstall.exe" "%s"';
begin
  TMB_BASLA.Enabled:=False;
// n Kontrol yapiliyor...
  OptionsHandle := 0;
  SourceDir := ExtractFilePath(ParamStr(0));
  isc_install_set_option(@OptionsHandle,INTERBASE);
  err_no := isc_install_precheck(OptionsHandle, PChar(SourceDir), nil);
  if (err_no > 0) then
    DisplayErrorMessage(OptionsHandle, err_no);
// Hedef Dizin alnyor....

  TmpDestDir  := StrAlloc(ISC_INSTALL_MAX_PATH);
  err_no :=  isc_install_get_info(isc_install_info_destination, 0, TmpDestDir, ISC_INSTALL_MAX_PATH);
  if(err_no = isc_install_success) then
    VAR_DESTINATION_DIR:=String(TmpDestDir)
  else
    FRM_INSTALL.DisplayErrorMessage(OptionsHandle, err_no);
  StrDispose(TmpDestDir);
// Kurulacak bilesenler set ediliyor...
  isc_install_clear_options(@OptionsHandle);
  for i := 0 to 1 do
  begin
    err_no := isc_install_set_option(@OptionsHandle, ComponentOptionNumber[i]);
    if (err_no <> 0) then
      DisplayErrorMessage(OptionsHandle, err_no);
  end;
// Dosyalar ykleniyor....

  UninstallFilename := StrAlloc(256);
  ret_val := mrIgnore;
  err_no := isc_install_precheck(OptionsHandle,PChar(SourceDir), PChar(VAR_DESTINATION_DIR));
  if (err_no <> 0) then
    ret_val := DisplayErrorMessage(OptionsHandle, err_no);
  if (ret_val = mrIgnore) then
  begin
    err_no := isc_install_execute(OptionsHandle,PChar(SourceDir), PChar(VAR_DESTINATION_DIR), @DisplayStatus,nil,@DisplayError,nil,UninstallFileName);
    if err_no = isc_install_user_aborted then
      exit;
    if (err_no <> 0) then
      ret_val := DisplayErrorMessage(OptionsHandle, err_no);
    if not (ret_val = mrIgnore) then
      exit;
  end;
  VAR_UNINSTALL_FILE_NAME := UninstallFilename;
  StrDispose(UninstallFilename);
  CopyFile(pChar(SourceDir+'\IB_Uninstall.exe'), pchar(VAR_DESTINATION_DIR+'\IB_Uninstall.exe'),False);
{
  if (RebootMessage) then
    MessageDlg('Ykleme ileminin bitmesi iin bilgisayarinizi yeniden balatmalsnz...',  MtInformation,[mbok],0)
  else
    MessageDlg('Ykleme baar ile tamamland!',MtInformation,[mbok],0);
}

// Interbase start ediliyor...
  verInfo.dwOSVersionInfoSize := sizeOf(verInfo);
  GetVersionEx(verInfo);
  if (verinfo.dwPlatformId = VER_PLATFORM_WIN32_NT) then
  begin
    ServiceArgs := nil;
    hManager := OpenSCManager(nil,nil,SC_MANAGER_CONNECT);
    hService := OpenService(hManager,GUARDIAN_SERVICE_NAME,SERVICE_START);
    if (hService = 0) then
      MessageDlg('Interbase Server balatlamyor...',mtError,[mbOk],0)
{    else
      if not StartService(hService, 0, ServiceArgs) then
        if not GetLastError() = ERROR_SERVICE_ALREADY_RUNNING then
          MessageDlg('Interbase Server balatlamyor...',mtError,[mbOk],0);
}          
    CloseServiceHandle(hManager);
    CloseServiceHandle(hService);
  end
//  If we are on 95/98 then we must create the process.
  else
  begin
    StartupInfo.cb := sizeof(StartupInfo);
    StartupInfo.lpReserved := nil;
    StartupInfo.lpTitle:= nil;
    StartupInfo.lpDesktop := nil;
    StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartupInfo.wShowWindow := SW_SHOWNA;
    StartupInfo.cbReserved2 := 0;
    StartupInfo.lpReserved2 := nil;
    if not CreateProcess(nil,PChar(Format(GUARDIAN_APPLICATION_NAME, [VAR_DESTINATION_DIR])), nil,nil,False,NORMAL_PRIORITY_CLASS,nil,
                         PChar(VAR_DESTINATION_DIR),StartupInfo,ProcessInformation) then
      MessageDlg('Interbase Server balatlamyor...',mtError,[mbOk],0);
  end;

// Uninstall programi register ediliyor...
  security := KEY_ALL_ACCESS;
  retval := RegOpenKeyEx(HKEY_LOCAL_MACHINE,RegUninstallKey,0,security,regHndl);
  if not (retval = ERROR_SUCCESS) then
    retval := RegCreateKeyEx(HKEY_LOCAL_MACHINE,RegUninstallKey,0,'',
  REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,regHndl,@KeyMade);
  if (retval = ERROR_SUCCESS) or (KeyMade = REG_CREATED_NEW_KEY) then
  begin
    if not (RegSetValueEx(regHndl,'DisplayName',0, REG_SZ,PChar(RegUninstallName), length(RegUninstallName)+1) = ERROR_SUCCESS) then
      MessageDlg('Uninstal program kaydedilemedi.',mtError,[mbOk],0);
    UninstallString := Format(RegUninstallString,[VAR_DESTINATION_DIR,VAR_UNINSTALL_FILE_NAME]);
    if not (RegSetValueEx(regHndl,'UninstallString', 0,REG_SZ,PChar(UninstallString), length(UninstallString)+1) = ERROR_SUCCESS) then
      MessageDlg('Uninstal program kaydedilemedi.',mtError,[mbOk],0)
  end
  else
    MessageDlg('Uninstal program kaydedilemedi.',mtError,[mbOk],0);
  RegCloseKey(regHndl);
  if VAR_CALISTIRILACAK_DOSYA_ADI<>'' then
    ExecuteFile(VAR_CALISTIRILACAK_DOSYA_ADI,'','',SW_SHOW);
  Self.Close;
end;

procedure TFRM_INSTALL.FormShow(Sender: TObject);
begin
  TMB_BASLA.Enabled:=True;
end;

procedure TFRM_INSTALL.FormCreate(Sender: TObject);
begin
  if ParamCount >= 1 then
    VAR_CALISTIRILACAK_DOSYA_ADI := ParamStr(1)
  else
    VAR_CALISTIRILACAK_DOSYA_ADI:='';
end;

end.
