unit FM_UNINSTALL;

interface

uses
Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
Gauges, StdCtrls, IBInstall, ComCtrls, ShlObj, ExtCtrls,WinSvc;


type
  TFRM_UNINSTALL = class(TForm)
    UninstallProgressGauge: TGauge;
    TMR_BASLA: TTimer;
    procedure TMR_BASLATimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);

  private

{ Private declarations }
    RebootMessage: Boolean;
    Cancelling: Boolean;
    function DisplayErrorMessage(error: MSG_NO): word;
    function StopGuardianService(servicename : string; timeout : integer) : boolean;
    function StopGuardianApp : boolean;

public

{ Public declarations }

end;

function DisplayError(
   msg: MSG_NO;
   data: pointer;
   error_msg: TEXT): integer; export; stdcall;

function DisplayStatus(
   status: integer;
   data: pointer;
   const status_msg: TEXT): integer; export; stdcall;

var

  FRM_UNINSTALL: TFRM_UNINSTALL;
  VAR_UNINSTALL_FILE_NAME:String;

implementation

{$R *.DFM}


function TFRM_UNINSTALL.DisplayErrorMessage(error: MSG_NO): word;
var
  msg: PChar;
begin
  msg := StrAlloc(ISC_INSTALL_MAX_MESSAGE_LEN);
  isc_install_get_message(0, error, msg, ISC_INSTALL_MAX_MESSAGE_LEN);
  if (error < isc_install_success) then
    result := MessageDlg(Strpas(msg),mtWarning,[mbOK, mbIgnore],0)
  else
    result := MessageDlg(Strpas(msg),mtWarning,[mbOK],0);
  StrDispose(msg);
end;

function DisplayError(msg: MSG_NO; data: pointer; error_msg: TEXT): integer;
var
  retval: word;
begin
  if not FRM_UNINSTALL.Cancelling then
  begin
    retval := mrIgnore;
    if (msg = isc_install_fp_copy_delayed) or (msg = isc_install_fp_delete_delayed) then
      FRM_UNINSTALL.RebootMessage := true
    else
      retval := FRM_UNINSTALL.DisplayErrorMessage(msg);
    if retval = mrIgnore then
      result := isc_install_fp_continue
    else
    begin
      FRM_UNINSTALL.Cancelling := true;
      result := isc_install_fp_abort;
    end;
  end
  else
    result := isc_install_fp_abort;
end;

function DisplayStatus(
   status: integer;
   data: pointer;
   const status_msg: TEXT): integer;
begin
  FRM_UNINSTALL.UninstallProgressGauge.Progress := status;
  Application.ProcessMessages;
  result := isc_install_fp_continue;
end;

procedure TFRM_UNINSTALL.TMR_BASLATimer(Sender: TObject);
const
  RootShortcutNames = 'Communication Diagnostics'+#10+
                      'InterBase Configuration Tool'+#10+
                      'InterBase Guardian'+#10+
                      'InterBase Server'+#10+
                      'License Registration Tool'+#10+
                      'Server Manager'+#10+
                      'InterBase Windows ISQL'+#10+
                      'Release Notes'+#10+
                      'License Agreement';
  DocsShortcutNames = 'InterBase Help'+#10+
                      'Performance Help'+#10+
                      'API Reference Guide'+#10+
                      'Data Definition Guide'+#10+
                      'Language Reference'+#10+
                      'Operations Guide'+#10+
                      'Programmer''s Guide'+#10+
                      'InterBase Tutorial';
var
  verInfo: TOSVersionInfo;
  StartMenuDir: pChar;
  StartFolderPath: String;
  ppidl: pItemIDList;
  SLinkNamesList: TStringList;
  i: integer;
  err_no: MSG_NO;
  security : REGSAM;
  regHndl  : HKey;
  retval: longint;
begin
  TMR_BASLA.Enabled:=False;
// Interbase durduruluyor...
  try
    if not StopGuardianApp then
    try
      StopGuardianService('InterbaseGuardian', 1000);
    except
    end;
  except
    try
      StopGuardianService('InterbaseGuardian', 1000);
    except
    end;
  end;
// Interbase kaldrlyor...
  err_no := isc_uninstall_execute(PChar(VAR_UNINSTALL_FILE_NAME), @DisplayStatus, nil, @DisplayError, nil);
  if (err_no <> isc_install_success) then
    DisplayErrorMessage(error);
  if (RebootMessage) then
    MessageDlg('Programn kaldrlma ileminin bitirilebilmesi iin bilgisayarnz yeniden balatmalsnz.', mtInformation, [mbok], 0)
  else
    MessageDlg('Program baarl bir ekilde kaldrld.',mtInformation, [mbok], 0);
// Program ekle kaldir listesinden kaldrlyor...
  security := KEY_ALL_ACCESS;
  retval := RegOpenKeyEx(HKEY_LOCAL_MACHINE,nil,0,security,regHndl);
  if (retval = ERROR_SUCCESS) then
    RegDeleteKey(HKEY_LOCAL_MACHINE,'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\InterBase');
  RegCloseKey(regHndl);
// Shortcut'lar kaldrlyor...
  verInfo.dwOSVersionInfoSize := sizeOf(verInfo);
  GetVersionEx(verInfo);
  if (verinfo.dwPlatformId = VER_PLATFORM_WIN32_NT) then
    SHGetSpecialFolderLocation(0, CSIDL_COMMON_PROGRAMS, ppidl)
  else
    SHGetSpecialFolderLocation(0, CSIDL_PROGRAMS, ppidl);
  StartMenuDir := StrAlloc(MAX_PATH);
  SHGetPathFromIDList(ppidl, StartMenuDir);
  StartFolderPath := StrPas(StartMenuDir) + '\InterBase';
  StrDispose (StartMenuDir);
  SLinkNamesList := TStringList.create;
  SLinkNamesList.text := RootShortcutNames;
  for i := 0 to SLinkNamesList.count - 1 do
    DeleteFile(StartFolderPath + '\' +SLinkNamesList.Strings[i] + '.lnk');
  SLinkNamesList.text := DocsShortcutNames;
  for i := 0 to SLinkNamesList.count - 1 do
    DeleteFile(StartFolderPath + '\Documentation\' + SLinkNamesList.Strings[i] + '.lnk');
  RemoveDir(StartFolderPath + '\Documentation');
  RemoveDir(StartFolderPath);
  Self.Close;
end;

procedure TFRM_UNINSTALL.FormShow(Sender: TObject);
begin
  TMR_BASLA.Enabled:=True;
end;

procedure TFRM_UNINSTALL.FormCreate(Sender: TObject);
begin
  if ParamCount >= 1 then
    VAR_UNINSTALL_FILE_NAME := ParamStr(1);
end;


function TFRM_UNINSTALL.StopGuardianService(servicename : string; timeout : integer) : boolean;
var
  scmanagerhandle, servicehandle : SC_HANDLE;
  servicestatus : SERVICE_STATUS;
begin
  Result := false;
  scmanagerhandle := OPenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if scmanagerhandle = 0 then
    MessageDlg('OpenSCManager failed', mtError, [mbOk], 0)
  else
  begin
    servicehandle := Openservice(scmanagerhandle, PChar(servicename),SERVICE_ALL_ACCESS);
    if servicehandle = 0 then
      MessageDlg('OpenService failed', mtError, [mbOk], 0)
    else
    begin
      if not ControlService(servicehandle, SERVICE_CONTROL_STOP, servicestatus) then
        MessageDlg('Couldn''t stop service', mtError, [mbOk], 0)
      else
      begin
        Result := true;
        Sleep(timeout);
      end;
      CloseServiceHandle(servicehandle);
    end;
    CloseServiceHandle(scmanagerhandle);
  end;
end;

function TFRM_UNINSTALL.StopGuardianApp : boolean;
var
  windowtext : string;
  textlength : integer;
  res1, res2 : boolean;
  hhwnd : HWND;
begin
  res1 := false;
  res2 := false;
//  get first Window handle in Z-control, which contains all open window handles
  hhwnd := GetWindow(Application.Handle, GW_HWNDFIRST);
  repeat
//  go through loop, test if handle is a Window handle and then check the window text to see which app it is.
//   it would seem that when you stop the guardian this way, you have to stop the server seperately, which is also handled in this code
    if IsWindow(hhwnd) then
    begin
      setlength(windowtext, 255);
      textlength := GetWindowText(hhwnd, PChar(windowtext), 255);
      setlength(windowtext, textlength);
      if textlength > 0 then
      begin
        windowtext := uppercase(windowtext);
        if (windowtext = 'INTERBASE GUARDIAN') then
          res1 := PostMessage(hhwnd, WM_CLOSE, 0, 0);
        if  (windowtext = 'INTERBASE SERVER') then
          res2 := PostMessage(hhwnd, WM_CLOSE, 0, 0);
      end;
    end;
    hhwnd := GetNextWindow(hhwnd, GW_HWNDNEXT);
  until hhwnd = 0;
  Result := res1 and res2;
end;


end.
